@@ -1025,6 +1025,18 @@ atspi_role_get_type (void)
 			{ ATSPI_ROLE_STATIC,
 			  "ATSPI_ROLE_STATIC",
 			  "static" },
+			{ ATSPI_ROLE_MATH_FRACTION,
+			  "ATSPI_ROLE_MATH_FRACTION",
+			  "math-fraction" },
+			{ ATSPI_ROLE_MATH_ROOT,
+			  "ATSPI_ROLE_MATH_ROOT",
+			  "math-root" },
+			{ ATSPI_ROLE_SUBSCRIPT,
+			  "ATSPI_ROLE_SUBSCRIPT",
+			  "subscript" },
+			{ ATSPI_ROLE_SUPERSCRIPT,
+			  "ATSPI_ROLE_SUPERSCRIPT",
+			  "superscript" },
 			{ ATSPI_ROLE_LAST_DEFINED,
 			  "ATSPI_ROLE_LAST_DEFINED",
 			  "last-defined" },
