@@ -521,11 +521,8 @@ already_running ()
   if (data)
   {
     GDBusConnection *bus;
-    const gchar *old_session = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
-    /* TODO: Is there a better way to connect? This is really hacky */
-    g_setenv ("DBUS_SESSION_BUS_ADDRESS", (const gchar *)data, TRUE);
-    bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
-    g_setenv ("DBUS_SESSION_BUS_ADDRESS", old_session, TRUE);
+    bus = g_dbus_connection_new_for_address_sync ((const gchar *)data, 0,
+                                                  NULL, NULL, NULL);
     if (bus != NULL)
       {
         result = TRUE;
